package com.sun.xml.internal.ws.client;

import java.util.Map;
import com.sun.xml.internal.ws.util.CompletedFuture;
import javax.xml.ws.WebServiceException;
import com.sun.istack.internal.Nullable;
import com.sun.xml.internal.ws.api.Cancelable;
import javax.xml.ws.AsyncHandler;
import javax.xml.ws.Response;
import java.util.concurrent.FutureTask;

public final class AsyncResponseImpl<T> extends FutureTask<T> implements Response<T>, ResponseContextReceiver
{
    private final AsyncHandler<T> handler;
    private ResponseContext responseContext;
    private final Runnable callable;
    private Cancelable cancelable;
    
    public AsyncResponseImpl(Runnable runnable, @Nullable AsyncHandler<T> handler) {
        super(runnable, null);
        this.callable = runnable;
        this.handler = handler;
    }
    
    @Override
    public void run() {
        try {
            this.callable.run();
        }
        catch (WebServiceException e) {
            this.set(null, e);
        }
        catch (Throwable e2) {
            this.set(null, new WebServiceException(e2));
        }
    }
    
    public ResponseContext getContext() {
        return this.responseContext;
    }
    
    public void setResponseContext(ResponseContext rc) {
        this.responseContext = rc;
    }
    
    public void set(T v, Throwable t) {
        if (this.handler != null) {
            try {
                class CallbackFuture<T> extends CompletedFuture<T> implements Response<T>
                {
                    public CallbackFuture(Throwable t) {
                        super(v, t);
                    }
                    
                    public Map<String, Object> getContext() {
                        return AsyncResponseImpl.this.getContext();
                    }
                }
                this.handler.handleResponse(new CallbackFuture(v, t));
            }
            catch (Throwable e) {
                super.setException(e);
                return;
            }
        }
        if (t != null) {
            super.setException(t);
        }
        else {
            super.set(v);
        }
    }
    
    public void setCancelable(Cancelable cancelable) {
        this.cancelable = cancelable;
    }
    
    @Override
    public boolean cancel(boolean mayInterruptIfRunning) {
        if (this.cancelable != null) {
            this.cancelable.cancel(mayInterruptIfRunning);
        }
        return super.cancel(mayInterruptIfRunning);
    }
}
